-----------------------------------------------------------------------------------------------
-- Client Lua Script for DefaultMount
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- DefaultMount Module Definition
-----------------------------------------------------------------------------------------------
local DefaultMount = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
local kcrSelectedText = ApolloColor.new("UI_BtnTextHoloPressedFlyby")
local kcrNormalText = ApolloColor.new("UI_BtnTextHoloNormal")
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function DefaultMount:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self 

    -- initialize variables here
  Print("init")
	o.tItems = {} -- keep track of all the list items
	o.wndSelectedListItem = nil -- keep track of which list item is currently selected

  return o
end

function DefaultMount:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    
  Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- DefaultMount OnLoad
-----------------------------------------------------------------------------------------------
function DefaultMount:OnLoad()
  -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("DefaultMount.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- DefaultMount OnDocLoaded
-----------------------------------------------------------------------------------------------
function DefaultMount:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "DefaultMountForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		-- item list
		self.wndItemList = self.wndMain:FindChild("ItemList")
    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("dm", "OnDefaultMountOn", self)

		-- Do additional Addon initialization here
    if self.defaultMount ~= nil then
      self:SetDefaultMount(self.defaultMount)
    end
	end
end

-----------------------------------------------------------------------------------------------
-- DefaultMount Save/Load
-----------------------------------------------------------------------------------------------
function DefaultMount:OnSave(eLevel)
  if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
    return nil
  end

  local tSavedData = { }

  self.defaultMount = self.wndSelectedListItem:GetData().tTiers[1].splObject
  
  tSavedData = {
    defaultMount = self.defaultMount:GetId()
  }

  return tSavedData
end

function DefaultMount:OnRestore(eLevel, tSavedData)
  if tSavedData ~= nil then
    self.defaultMount = tSavedData.defaultMount
  end
end

-----------------------------------------------------------------------------------------------
-- DefaultMount Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function DefaultMount:SetDefaultMount(mount)
  Apollo.GetAddon("ActionBarFrame").nSelectedMount = mount
  Apollo.GetAddon("ActionBarFrame"):RedrawMounts()
end

-- on SlashCommand "/dm"
function DefaultMount:OnDefaultMountOn()
	self.wndMain:Invoke() -- show the window
	self:PopulateItemList()
end

function DefaultMount:OnOK()
	self.wndMain:Close()
end

function DefaultMount:OnCancel()
  self.wndMain:Close()
end

function DefaultMount:PopulateItemList()
	-- make sure the item list is empty to start with
	self:DestroyItemList()
	
  local tMountList = AbilityBook.GetAbilitiesList(Spell.CodeEnumSpellTag.Mount) or {}
  for idx, tMountData  in pairs(tMountList) do
    self:AddMount(tMountData)
  end
	
	-- now all the item are added, call ArrangeChildrenVert to list out the list items vertically
	self.wndItemList:ArrangeChildrenVert()
end

-- clear the item list
function DefaultMount:DestroyItemList()
  for idx,wnd in pairs(self.tItems) do
    wnd:Destroy()
  end

	-- clear the list item array
	self.tItems = {}
	self.wndSelectedListItem = nil
end

function DefaultMount:AddMount(mount)
  local wnd = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndItemList, self)

  self.tItems[mount.strName] = wnd

  local wndItemText = wnd:FindChild("Text")
  local wndItemIcon = wnd:FindChild("MountIcon")
  if wndItemText then -- make sure the text wnd exist
    wndItemText:SetText(mount.strName)
    wndItemText:SetTextColor(kcrNormalText)
  end

  if wndItemIcon then
    wndItemIcon:SetSprite(mount.tTiers[1].splObject:GetIcon())
  end

  wnd:SetData(mount)
end

-- when a list item is selected
function DefaultMount:OnListItemSelected(wndHandler, wndControl)
  -- make sure the wndControl is valid
  if wndHandler ~= wndControl then
    return
  end
  
  -- change the old item's text color back to normal color
  local wndItemText
  if self.wndSelectedListItem ~= nil then
    wndItemText = self.wndSelectedListItem:FindChild("Text")
    wndItemText:SetTextColor(kcrNormalText)
  end
    
	-- wndControl is the item selected - change its color to selected
	self.wndSelectedListItem = wndControl
	wndItemText = self.wndSelectedListItem:FindChild("Text")
  wndItemText:SetTextColor(kcrSelectedText)

  local selection = self.wndSelectedListItem:GetData().tTiers[1].splObject:GetId()

	self.defaultMount = selection
  self:SetDefaultMount(selection)
end

-----------------------------------------------------------------------------------------------
-- DefaultMount Instance
-----------------------------------------------------------------------------------------------
local DefaultMountInst = DefaultMount:new()
DefaultMountInst:Init()
